﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.IO;

namespace Deleter
{
    class FileMunger
    {

        private AppParams _appParams;
        private bool _delete;

        /// <summary>
        /// Params have been validated.
        /// </summary>
        public FileMunger( AppParams appParams )
        {
            _appParams = appParams;
            _appParams.Cutoff = ( _appParams.Cutoff.HasValue ) ? _appParams.Cutoff.Value : DateTime.Now.AddDays( -30 );
            _delete = _appParams.OutputPath == null;
        }

        private IEnumerable<FileInfo> GetFiles( string dir )
        {
            return FilterApplicator.Apply( FileUtils.GetFiles( dir ), _appParams );
        }

        private IEnumerable<string> GetFolders( string dir )
        {
            return FilterApplicator.Apply( Directory.EnumerateDirectories( dir ), _appParams );
        }

        public void Process()
        {
            if ( MungeGoods() )
            {
                if ( _delete )
                {
                    Console.WriteLine( "Finished." );
                }
                else
                {
                    Console.WriteLine( "Output file created: " + _appParams.OutputPath );
                }
            }
        }

        private bool MungeGoods()
        {
            IEnumerable<object> goods;

            if ( _appParams.FolderMode )
            {
                goods = GetFolders( _appParams.SearchDir );
            }
            else
            {
                goods = GetFiles( _appParams.SearchDir );
            }

            if ( goods.Count() == 0 )
            {
                const string NO_MATCH = "No matching files / folders.";
                if ( !_delete )
                {
                    FileUtils.stringToFile( NO_MATCH, _appParams.OutputPath, true, Encoding.ASCII );
                }
                Console.WriteLine( NO_MATCH );
                return false;
            }
            else
            {
                if ( _appParams.FolderMode )
                {
                    MungeFolders( (IEnumerable<string>)goods );
                }
                else
                {
                    MungeFiles( (IEnumerable<FileInfo>)goods );
                }
            }

            return true;
        }

        private void MungeFolders( IEnumerable<string> folders )
        {
            folders.ToList().ForEach( f =>
            {
                if ( _delete )
                {
                    try
                    {
                        FileUtils.FlushDirectory( f, true );
                    }
                    catch ( Exception )
                    {
                        WriteDeletionErr( f );
                        return;
                    }
                }
                else
                {
                    WriteCommandToFile( f );
                }
            } );
        }

        private void MungeFiles( IEnumerable<FileInfo> files )
        {
            foreach ( var s in files )
            {
                var fileName = s.FullName;
                if ( _delete )
                {
                    try
                    {
                        FileUtils.DeleteFile( fileName );
                    }
                    catch ( Exception )
                    {
                        WriteDeletionErr( fileName );
                        break;
                    }
                }
                else
                {
                    WriteCommandToFile( fileName );
                }
            }
        }

        private void WriteCommandToFile( string objectName )
        {
            string line;
            if ( _appParams.FolderMode )
            {
                line = string.Format( "rd /Q /S {0}{1}{0}{2}", '"', objectName, Environment.NewLine );
            }
            else
            {
                line = string.Format( "del /Q {0}{1}{0}{2}", '"', objectName, Environment.NewLine );
            }
            FileUtils.stringToFile( line, _appParams.OutputPath, true, Encoding.ASCII );
        }

        private void WriteDeletionErr( string objectCausingError )
        {
            Console.WriteLine( "Unable to delete: " + objectCausingError );
            Console.WriteLine( "Terminating process." );
        }

    }
}
