﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Collections.Specialized;

namespace Deleter
{
    class FileUtils
    {
        
        /// <summary>
        /// UTF-7: handles only base ASCII
        /// UTF-8: handles extended ASCII.  You may encode the Unicode set (double bytes) with
        ///        this encoding.  It uses only single bytes when necessary.
        /// UTF-16: Every byte is double
        /// </summary>
        /// <param name="s">String to write</param>
        /// <param name="spec">Path to write to</param>
        /// <param name="append">Whether to append to existing file</param>
        /// <param name="encoding">How to encode</param>
        /// <returns>success</returns>
        public static bool stringToFile( string s, string spec, bool append, Encoding encoding )
        {
            try
            {
                using ( StreamWriter sw = new StreamWriter( spec, append, encoding ) )
                {
                    sw.Write( s );
                }
                return true;
            }
            catch
            {
                return false;
            }
        }

        public static string fileToString( string spec, Encoding encoding )
        {
            try
            {
                using ( StreamReader sr = new StreamReader( spec, encoding ) )
                {
                    return sr.ReadToEnd();
                }
            }
            catch
            {
                return string.Empty;
            }
        }

        /// <summary>
        /// Returns FileInfo instead of strings as framework call does
        /// </summary>
        public static IEnumerable<System.IO.FileInfo> GetFiles( string path )
        {
            if ( !System.IO.Directory.Exists( path ) )
                throw new System.IO.DirectoryNotFoundException();

            List<System.IO.FileInfo> files = new List<System.IO.FileInfo>();
            string[] fileNames = System.IO.Directory.GetFiles( path, "*.*", System.IO.SearchOption.AllDirectories );

            foreach ( string name in fileNames )
            {
                files.Add( new System.IO.FileInfo( name ) );
            }

            return files;
        }

        /// <summary>
        /// Clear the frigging (read-only) attribute so you can delete the @#$*ers
        /// </summary>
        /// <param name="currentDir"></param>
        public static void ClearAttributes( string currentDir )
        {
            if ( Directory.Exists( currentDir ) )
            {
                string[] subDirs = Directory.GetDirectories( currentDir );
                foreach ( string dir in subDirs )
                {
                    ClearAttributes( dir );
                }
                string[] files = files = Directory.GetFiles( currentDir );
                foreach ( string file in files )
                {
                    File.SetAttributes( file, FileAttributes.Normal );
                }
            }
        }

        /// <summary>
        /// Can't just call the overload to Directory.Delete that is recursive because we need to call ClearAttributes on every frigging file or the 
        /// son-of-a-bitch won't delete.
        /// </summary>
        /// <param name="currentDir"></param>
        /// <param name="deleteDirAfterFlush"></param>
        public static void FlushDirectory( string currentDir, bool deleteDirAfterFlush = false )
        {
            FlushFilesFromDirectory( currentDir );
            var subDirs = Directory.EnumerateDirectories( currentDir );
            foreach ( var dir in subDirs )
            {
                FlushDirectory( dir, deleteDirAfterFlush );
            }
            if ( deleteDirAfterFlush )
            {
                DeleteDirectory( currentDir );
            }
        }

        public static void DeleteFile( string filePath )
        {
            File.SetAttributes( filePath, FileAttributes.Normal );
            File.Delete( filePath );
        }

        /// <summary>
        /// Here's the story:
        /// If you've just deleted the files from a directory, many times (usually) you can't delete the directory immediately afterwards; windows
        /// will report that the directory isn't empty.  Thus, here we try for up to 2 seconds and if it's still failing after that we stop trying.
        /// </summary>
        /// <param name="currentDir"></param>
        private static void DeleteDirectory( string currentDir )
        {
            var ticks = Environment.TickCount;
            while ( ( Environment.TickCount - ticks ) < 3000 )
            {
                try
                {
                    Directory.Delete( currentDir );
                    return;
                }
                catch ( Exception )
                {
                    if ( ( Environment.TickCount - ticks ) > 2000 )
                    {
                        throw;
                    }
                }
            }
        }

        private static void FlushFilesFromDirectory( string currentDir )
        {
            ClearAttributes( currentDir );
            EnumerateFiles( currentDir ).ToList().ForEach( f => File.Delete( f ) );
        }

        /// <summary>
        /// Wrapper around framework method with params to get all files
        /// </summary>
        /// <param name="dir"></param>
        /// <returns></returns>
        private static IEnumerable<string> EnumerateFiles( string dir )
        {
            return Directory.EnumerateFiles( dir, "*.*", SearchOption.AllDirectories );
        }

    }
}
