﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace Deleter
{

    /// <summary>
    /// There's a filter each for files/folders
    /// </summary>
    public static class Filters
    {

        #region Filter by date

        public static IEnumerable<FileInfo> FilterByDate( this IEnumerable<FileInfo> files, DateTime dt )
        {
            return files.Where( f => FilterByDate( dt, f.CreationTime ) );
        }

        public static IEnumerable<string> FilterByDate( this IEnumerable<string> folders, DateTime dt )
        {
            return folders.Where( f => FilterByDate( dt, Directory.GetCreationTime( f ) ) );
        }

        private static bool FilterByDate( DateTime cutoff, DateTime creation )
        {
            return DateTime.Compare( cutoff, creation ) > 0;
        }

        #endregion

        #region Filter by name begins with numeric

        public static IEnumerable<FileInfo> FilterByThoseBeginningWithNumeric( this IEnumerable<FileInfo> files )
        {
            return files.Where( f => BeginsWithNumeric( f.Name ) );
        }

        public static IEnumerable<string> FilterByThoseBeginningWithNumeric( this IEnumerable<string> folders )
        {
            return folders.Where( f => BeginsWithNumeric( f ) );
        }

        private static bool BeginsWithNumeric( string name )
        {
            int dummy;
            return int.TryParse( name.Substring( name.LastIndexOf( Path.DirectorySeparatorChar ) + 1, 1 ), out dummy );
        }

        #endregion

        #region Filter by name is guid

        public static IEnumerable<FileInfo> FilterByNameIsGuid( this IEnumerable<FileInfo> files )
        {
            return files.Where( f => NameIsGuid( f.Name ) );
        }

        public static IEnumerable<string> FilterByNameIsGuid( this IEnumerable<string> folders )
        {
            return folders.Where( f =>
            {
                var dirName = f.Substring( f.LastIndexOf( '\\' ) + 1 );
                return NameIsGuid( dirName );
            } );
        }

        private static bool NameIsGuid( string name )
        {
            Guid dummy;
            return Guid.TryParse( name, out dummy );
        }

        #endregion

    }
}
